﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Ria.ApplicationServices
Imports System.Windows.Ria.Data
Imports System.Windows.Shapes
Imports AlexApp
Imports ExpenseReport

Namespace UserControls
	''' <summary>
	''' Login Control that allows a user to login and logout and displays the users name.
	''' </summary>
	Partial Public Class LoginControl
		Inherits UserControl
		Private parentApp As ExpenseReportApp = Nothing
		Private loginWindow_Renamed As LoginChildWindow = Nothing
		Private domainContext As ExpenseData = Nothing

		Public Sub New()
			InitializeComponent()

			domainContext = New ExpenseData()

			CheckLoginStatus()
			AddHandler UserService.Current.LoginCompleted, AddressOf UserService_LoginCompleted
			AddHandler UserService.Current.LogoutCompleted, AddressOf UserService_LogoutCompleted
		End Sub

		''' <summary>
		''' Helper property that creates a LoginChild window if one hasn't been created.
		''' </summary>
		Private ReadOnly Property LoginWindow() As LoginChildWindow
			Get
				If loginWindow_Renamed Is Nothing Then
					loginWindow_Renamed = New LoginChildWindow()
					AddHandler loginWindow_Renamed.userIdTextBox.TextChanged, AddressOf userIdTextBox_TextChanged
					AddHandler loginWindow_Renamed.loginFormButton.Click, AddressOf loginFormButton_Click
					AddHandler loginWindow_Renamed.cancelFormButton.Click, AddressOf cancelFormButton_Click
				End If
				Return loginWindow_Renamed
			End Get
		End Property

		''' <summary>
		''' Check to see if current user is logged in an set the visual state of the control.
		''' </summary>
		Public Sub CheckLoginStatus()
			Dim user As User = TryCast(UserService.Current.User, User)
			Me.LoginButton.IsEnabled = True

			'Find parent application for navigation.
			Dim element As UIElement = Me
			Do While element IsNot Nothing AndAlso element.GetType() IsNot GetType(ExpenseReportApp)
				element = CType(VisualTreeHelper.GetParent(element), UIElement)
			Loop
			If element IsNot Nothing Then
				parentApp = TryCast(element, ExpenseReportApp)
			End If

			If user IsNot Nothing AndAlso user.IsAuthenticated Then
				LoginButton.Content = "Logout"
				LoginWindow.Close()
				WelcomeText.Visibility = Visibility.Visible

				'User is authenticated so get the current user's name.
				AddHandler domainContext.Loaded, AddressOf domainContext_Loaded
				domainContext.LoadEmployees(ExpenseData.EmployeesQuery.Where(Function(e) e.employee = UserService.Current.User.Name),Nothing)

				If element IsNot Nothing AndAlso parentApp IsNot Nothing AndAlso parentApp.myFrame IsNot Nothing Then
					parentApp.myFrame.Navigate(New Uri("/ExpenseReportSummary.xaml", UriKind.Relative))

				End If
			ElseIf user Is Nothing AndAlso parentApp IsNot Nothing AndAlso parentApp.myFrame IsNot Nothing Then
				parentApp.myFrame.Navigate(New Uri("/Instructions.xaml", UriKind.Relative))
			End If
		End Sub

		''' <summary>
		''' Responds to loaded event from DomainContext and sets the value of TextBlock to the employee's name. 
		''' </summary>
		''' <param name="sender">DomainContext</param>
		''' <param name="e">LoadedDataEventArgs for Loaded event</param>
		Private Sub domainContext_Loaded(ByVal sender As Object, ByVal e As LoadedDataEventArgs)
			AddHandler domainContext.Loaded, AddressOf domainContext_Loaded
			Dim emp As Employee = CType(e.LoadedEntities.FirstOrDefault(), Employee)
			UserIdTextBlock.Text = emp.Name
		End Sub

		''' <summary>
		''' Validates that users was correctly authenticated and updates visual elements.
		''' </summary>
		''' <param name="sender">UserService</param>
		''' <param name="e">LoginCompletedEventArgs for LoginCompleted event</param>
		Private Sub UserService_LoginCompleted(ByVal sender As Object, ByVal e As LoginCompletedEventArgs)
			Dim user As User = TryCast(UserService.Current.User, User)

			CheckLoginStatus()

			If user IsNot Nothing AndAlso (Not user.IsAuthenticated) Then
				loginWindow_Renamed.statusMessage.Text = "Login Failed please try again."
				loginWindow_Renamed.userIdTextBox.Text = ""
				loginWindow_Renamed.passwordTextBox.Password = ""
				WelcomeText.Visibility = Visibility.Collapsed
				UserIdTextBlock.Text = ""
			End If
		End Sub

		''' <summary>
		''' Responds to LogoutEvent and updates visual elements.
		''' </summary>
		''' <param name="sender">UserService</param>
		''' <param name="e">LogoutCompletedEventArgs for LogoutCompleted event</param>
		Private Sub UserService_LogoutCompleted(ByVal sender As Object, ByVal e As LogoutCompletedEventArgs)
			LoginButton.Content = "Login"
			WelcomeText.Visibility = Visibility.Collapsed
			UserIdTextBlock.Text = ""
			Dim currentUser As User = TryCast(UserService.Current.User, User)
			If currentUser IsNot Nothing And (Not currentUser.IsAuthenticated) AndAlso parentApp IsNot Nothing Then
				parentApp.myFrame.Navigate(New Uri("/Instructions.xaml", UriKind.Relative))

			End If
		End Sub

		''' <summary>
		''' Based on state of login button either logs out the user or shows the login window.
		''' </summary>
		''' <param name="sender">LoginButton</param>
		''' <param name="e">RoutedEventArgs for button's click event.</param>
		Private Sub LoginButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If String.Compare(LoginButton.Content.ToString(), "Login")=0 Then
				LoginWindow.Show()
			Else
				UserService.Current.Logout()
			End If
		End Sub

		''' <summary>
		''' Updates that status message when new text is entered for User ID
		''' </summary>
		''' <param name="sender">User ID textbox</param>
		''' <param name="e">TextChangedEventArgs for user ID textbox</param>
		Private Sub userIdTextBox_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
			'If user types in text then clear status message.
			Dim textBox As TextBox = CType(sender, TextBox)
			If textBox IsNot Nothing AndAlso textBox.Text <> "" Then
				loginWindow_Renamed.statusMessage.Text = ""
			End If
		End Sub

		''' <summary>
		''' Reacts to Login button clicked on LoginChildWindow.  
		''' </summary>
		''' <param name="sender">Login button</param>
		''' <param name="e">RoutedEventArgs for Login button click.</param>
		Private Sub loginFormButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If UserService.Current.IsBusy Then
				loginWindow_Renamed.statusMessage.Text = "Please wait for login to complete."
			Else
				UserService.Current.Login(New System.Windows.Ria.ApplicationServices.LoginParameters(loginWindow_Renamed.userIdTextBox.Text, loginWindow_Renamed.passwordTextBox.Password))
			End If
		End Sub

		''' <summary>
		''' Reacts to Cancel button clicked on LoginChildWindow.  
		''' </summary>
		''' <param name="sender">Cancel button</param>
		''' <param name="e">RoutedEventArgs for Cancel button click.</param>
		Private Sub cancelFormButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			LoginWindow.Close()
		End Sub

	End Class
End Namespace
