﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Data
Imports System.Data.Objects
Imports System.Linq
Imports System.Web.DomainServices
Imports System.Web.DomainServices.LinqToEntities
Imports System.Web.Ria
Imports System.Web.Ria.ApplicationServices

Namespace ExpenseReport
	'DataProvider to expose ExpenseReports and related entities
	<EnableClientAccess> _
	Public Class ExpenseData
		Inherits LinqToEntitiesDomainService(Of ExpenseReports)

		''' <summary>
		''' Query method that returns ExpenseReports for the current user and also 
		''' returns the related employee instance.
		''' </summary>
		''' <returns>ExpenseReports for the current employee and also returns the related employee instance.</returns>
		<RequiresAuthentication()> _
		Public Function GetExpenseReportsForUser() As IQueryable(Of ExpenseReport)
			Return Me.Context.ExpenseReport.Include("Employee").Where(Function(e) e.Employee.employee = UserService.Current.User.Name)
		End Function


		''' <summary>
		''' Query method that returns ExpenseReports have been Submitted and are ready for 
		''' Approval (Status is 1)and belong to an employee whose manager is the current user.
		''' </summary>
		''' <returns>ExpenseReports that have been Submitted and are ready for Approval (Status is 1)and 
		''' belong to an employee whose manager is the current employee.
		''' </returns>
		<RequiresRoles("manager")> _
		Public Function GetExpenseReportsForApproval() As IQueryable(Of ExpenseReport)
			Return Me.Context.ExpenseReport.Include("Employee").Where(Function(e) e.Employee.manager = UserService.Current.User.Name AndAlso e.Status = 1)
		End Function

		''' <summary>
		''' Query method that returns the ExpenseReport for the ID passed and referenced instances.
		''' </summary>
		''' <param name="erId">ID of ExpenseReport to be returned.</param>
		''' <returns>Returns ExpenseReport with ID passed in and that ExpenseReport's associated ExpenseReportDetails
		''' and Employee instances</returns>
		<RequiresAuthentication()> _
		Public Function GetExpenseReportWithDetails(ByVal erId As Integer) As IQueryable(Of ExpenseReport)
			Return Me.Context.ExpenseReport.Include("ExpenseReportDetails").Include("Employee").Where(Function(e) e.id = erId).Take(1)
		End Function

		''' <summary>
		''' Inserts ExpenseReport.
		''' </summary>
		''' <param name="er">ExpenseReport to insert.</param>
		<RequiresAuthentication()> _
		Public Sub InsertExpenseReport(ByVal er As ExpenseReport)
			Context.AddToExpenseReport(er)
		End Sub

		''' <summary>
		''' Updates an ExpenseReport.
		''' </summary>
		''' <param name="current">Updated ExpenseReport after changes were made.</param>
		''' <param name="original">Original ExpenseReport before any chagnes were made.</param>
		<RequiresAuthentication()> _
		Public Sub UpdateExpenseReport(ByVal current As ExpenseReport, ByVal original As ExpenseReport)
			Context.AttachAsModified(current, original)
		End Sub

		''' <summary>
		''' Deletes ExpenseReport.
		''' </summary>
		''' <param name="er">ExpenseReport to insert.</param>
		<RequiresAuthentication()> _
		Public Sub DeleteExpenseReport(ByVal er As ExpenseReport)
			'Checks to see it entity instance is attached because an instance cannot be attached twice.
			If er.EntityState = EntityState.Detached Then
				Context.Attach(er)
			End If
			Context.DeleteObject(er)
		End Sub

		''' <summary>
		''' Service operation that updates an expense reports status to the submitted state.
		''' </summary>
		''' <param name="er">ExpenseReport being submitted.</param>
		<ServiceOperation, RequiresAuthentication()> _
		Public Sub SubmitExpenseReport(ByVal er As ExpenseReport)
			If er.Status = 0 Then 'if Status = Not Submitted
				'Checks to see it entity instance is attached because an instance cannot be attached twice.
				If er.EntityState = System.Data.EntityState.Detached Then
					Context.Attach(er)
				End If
				er.Status = 1
			End If
		End Sub

		''' <summary>
		''' Service operation that updates an expense reports status to the approved state.
		''' </summary>
		''' <param name="er">ExpenseReport being approved.</param>
		<ServiceOperation, RequiresRoles("manager")> _
		Public Sub ApproveExpenseReport(ByVal er As ExpenseReport)
			If er.Status = 1 Then 'if Status = Submitted
				'Checks to see it entity instance is attached because an instance cannot be attached twice.
				If er.EntityState = System.Data.EntityState.Detached Then
					Context.Attach(er)
				End If
				er.Status = 2
			End If
		End Sub

		''' <summary>
		''' Service operation that updates an expense reports status to the rejected state.
		''' </summary>
		''' <param name="er">ExpenseReport being rejected.</param>
		<ServiceOperation, RequiresRoles("manager")> _
		Public Sub RejectExpenseReport(ByVal er As ExpenseReport)
			If er.Status = 1 Then
				If er.EntityState = System.Data.EntityState.Detached Then
					Context.Attach(er)
				End If
				er.Status = 0
			End If
		End Sub

		''' <summary>
		''' Query method that returns all expense report details where the expense report was created by
		''' the current users or the current user is the manager of the employee who created the expense report.
		''' </summary>
		''' <returns>All Expense Report Details that the current user has access to.</returns>
		<RequiresAuthentication()> _
		Public Function GetExpenseReportDetails() As IQueryable(Of ExpenseReportDetails)
			Return Me.Context.ExpenseReportDetails.Where(Function(e) e.ExpenseReport.Employee.employee = UserService.Current.User.Name OrElse e.ExpenseReport.Employee.manager = UserService.Current.User.Name)
		End Function

		''' <summary>
		''' Inserts ExpenseReportDetails.
		''' </summary>
		''' <param name="er">ExpenseReportDetails to insert.</param>
		<RequiresAuthentication()> _
		Public Sub InsertExpenseReportDetails(ByVal er As ExpenseReportDetails)
			Context.AddToExpenseReportDetails(er)
		End Sub

		''' <summary>
		''' Updates an ExpenseReportDetails.
		''' </summary>
		''' <param name="current">Updated ExpenseReportDetails after changes were made.</param>
		''' <param name="original">Original ExpenseReportDetails before any chagnes were made.</param>
		<RequiresAuthentication()> _
		Public Sub UpdateExpenseReportDetails(ByVal current As ExpenseReportDetails, ByVal original As ExpenseReportDetails)
			Context.AttachAsModified(current, original)
		End Sub

		''' <summary>
		''' Deletes ExpenseReportDetails.
		''' </summary>
		''' <param name="er">ExpenseReportDetails to delete.</param>
		<RequiresAuthentication()> _
		Public Sub DeleteExpenseReportDetails(ByVal er As ExpenseReportDetails)
			If er.EntityState = System.Data.EntityState.Detached Then
				Context.Attach(er)
			End If
			Context.DeleteObject(er)
		End Sub

		''' <summary>
		''' Query method to return all Employees.
		''' </summary>
		''' <returns>All Employees</returns>
		<RequiresAuthentication()> _
		Public Function GetEmployees() As IQueryable(Of Employee)
			Return Me.Context.Employee
		End Function
	End Class

End Namespace
